/**
   A BTree is defined by the following grammar.

     BTree ::=  '(' String BTree BTree ')'
             |  '(' String '()'  BTree ')'
             |  '(' String BTree  '()' ')'
             |  String

   where '(' and ')' and '()' are literals. This language also
   assumes that all tokens have white space around them.

   The static method getBTree() is essentially a recursive descent
   parser for this BTree langauge. Input strings are "tokenized" by
   the very bare bones tokenizer contained in the file Tokenizer.java.
   The requirement that all tokens in the input string be surrounded
   by white space makes the tokenizer almost trivial to write.
*/

public class BuildBTree
{
   public static BTree buildBTree(String expression)
   {
      Tokenizer tokens = new Tokenizer(expression);  // Tokenizer is defined
                                                     // in Tokenizer.java
      BTree result = getBTree(tokens); // parse the tokens

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new Error("syntax error: unexpected input: "+tokens.peekToken()+"\n"+tokens);

      return result;
   }//buildBTree()


   private static BTree getBTree(Tokenizer tokens)
   {
      BTree result;

      String token = tokens.nextToken(); // consume one token
      if ( token.equals("(") )           // look for a parenthesized tree
      {
         result = new BTree( tokens.nextToken() );   // consume the root of the tree
         if ( tokens.peekToken().equals("()") )
         {
            tokens.match("()");
            result.addRightTree( getBTree(tokens) );  // consume right sub tree
         }
         else
         {
            result.addLeftTree( getBTree(tokens) );   // consume left sub tree
            if ( tokens.peekToken().equals("()") )
            {
               tokens.match("()");
            }
            else
            {
               result.addRightTree( getBTree(tokens) ); // consume right sub tree
            }
         }
         if ( !tokens.match(")") )   // consume the matching ")"
            throw new Error("syntax error: expected \")\"" + "\n" + tokens);
      }
      else // the tree must be just the root
      {
         result = new BTree( token );
      }
      return result;
   }//getBTree()

}//BuildBTree
